/********************************************************************\
 * Account.h -- the Account data structure                          *
 * Copyright (C) 1997 Robin D. Clark                                *
 *                                                                  *
 * This program is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU General Public License as   *
 * published by the Free Software Foundation; either version 2 of   *
 * the License, or (at your option) any later version.              *
 *                                                                  *
 * This program is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU General Public License for more details.                     *
 *                                                                  *
 * You should have received a copy of the GNU General Public License*
 * along with this program; if not, write to the Free Software      *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 *                                                                  *
 *   Author: Rob Clark                                              *
 * Internet: rclark@cs.hmc.edu                                      *
 *  Address: 609 8th Street                                         *
 *           Huntington Beach, CA 92648-4632                        *
\********************************************************************/

#ifndef __ACCOUNT_H__
#define __ACCOUNT_H__

#include "QuickFill.h"
#include "Transaction.h"

/* The account types */
enum {
  BANK,
  CASH,
  ASSET,
  CREDIT,
  LIABILITY,
  PORTFOLIO,
  MUTUAL 
};

#define XACC_NUM_COLS  12

/** STRUCTS *********************************************************/
typedef struct _Account {
  /* public data, describes account */

  struct _data *data;         /* back-pointer to parent */
  int       id;               /* unique account id, internally assigned */
  char      flags;
  char      type;
  String    accountName;
  String    description;
  String    notes;

  /* protected data, cached parameters */
  double balance;
  double cleared_balance;

  int         numTrans;
  Transaction **transaction;    /* pointer to array of pointers to transactions */

  /* private data, controls display and editing */
  short columnLocation [XACC_NUM_COLS];
  short numCols;
  short colWidths [XACC_NUM_COLS];
  String rows [3][XACC_NUM_COLS];
  unsigned char alignments[XACC_NUM_COLS];

  struct _RegWindow  *regData;  /* the register window if open, or NULL    */
  struct _RecnWindow *recnData; /* the reconcile window, if open, or NULL  */
  struct _AdjBWindow *adjBData; /* the adjust-balance window, or NULL      */
  QuickFill   *qfRoot;          /* the quickfill tree for this Account     */
} Account;

/** PROTOTYPES ******************************************************/

Account     *mallocAccount( void );
void         freeAccount( Account *acc );
Transaction *getTransaction( Account *acc, int num );
Transaction *removeTransaction( Account *acc, int num );
int          insertTransaction( Account *acc, Transaction *trans );
int          getNumOfTransaction( Account *, Transaction *);

/* The following two functions set and get the amount on the transaction 
 * The account argument is needed to determine if this is a credit
 * or a debit, since this is a double-entry system */
double       xaccGetAmount (Account *, Transaction *);
double       xaccGetShareAmount (Account *, Transaction *);
void         xaccSetAmount (Account*, Transaction *, double);
void         xaccSetShareAmount (Account*, Transaction *, double);

/* the following recompute the partial balances (stored with the transaction)
 * and the total balance, for this account */
void         xaccRecomputeBalance (Account *);

/* The following two methods return the running balance up to the indicated 
 * transaction. */
double       xaccGetBalance (Account *, Transaction *);
double       xaccGetClearedBalance (Account *, Transaction *);
double       xaccGetShareBalance (Account *, Transaction *);

/* the following methods check the date order of the transacton */
int          xaccCheckDateOrder   (Account *, Transaction *);
int          xaccCheckDateOrderDE (Transaction *);


/** GLOBALS *********************************************************/

extern int next_free_unique_account_id;


#endif
